% Author: Kubra SEKMEN* & Eric PAROISSIEN & Frederic LACHAUD
% ISAE - SUPAERO Institut Suprieur de l'Aronautique et de l'Espace
% Date: 10.07.2019
% Paper Title: Simplified Stress Analysis of Multilayered Adhesively
%               Bonded Structures
% Code attached to the related paper submitted to AB2019
% ==> 1D Bar Model - Volkersen Simplified Hypotheses
% ==> in-plane tensile loading at the tip (x=L) of the last adherend layer
% LICENSE: CC BY NC 3.0 [https://creativecommons.org/licenses/by-nc/3.0/]
% =========================================================================
clear all
close all
clc
% ========================================================================
% A - PROBLEM DEFINITION
% ========================================================================
% =========================================================================
% A1 - SIZE OF THE PROBLEM
% =========================================================================
% =========================================================================

% P     number of adherend layers
P = 4;

% n_ME  number of macro-elements
n_ME = 100;

% DEDUCED PARAMETERS
n_E = n_ME;               % total number of elements  
n_nodes_ME = P*(n_ME+1);  % total number of nodes in ME
n_nodes = n_nodes_ME;     % total number of nodes     
n_DOF_ME = 1*n_nodes_ME;  % total number of DOF in ME
n_DOF = 1*n_nodes;        % total number of DOF

% =========================================================================
% A2 - GEOMETRY
% =========================================================================
% =========================================================================
% Geometric Parameters

% e adherend thickness        (mm)
% t adhesive thickness        (mm)
% b width of adherends        (mm) 
% L overlap length            (mm)

b = 1;            %mm	% adherend thickness
L = 30;           %mm   % overlap length    

% It is possible
% ==> to use different adherend thicknesses and 
% ==> to increase the number of adherends in the multilayered structure
e1 = 2.5;         %mm   
e2 = e1;          %mm
e3 = e1;          %mm
e4 = e1;          %mm
% e5 = 2.5;         %mm
% e6 = 2.5;         %mm
% e7 = 2.5;         %mm
% e8 = 2.5;         %mm
% e9 = 2.5;         %mm
% e10 = 2.5;        %mm

t1 = 0.11;        %mm
t2 = 0.11;        %mm
t3 = 0.11;        %mm
% t4 = 0.11;        %mm
% t5 = 0.11;        %mm
% t6 = 0.11;        %mm
% t7 = 0.11;        %mm
% t8 = 0.11;        %mm
% t9 = 0.11;        %mm
% t10 = 0.11;       %mm

% Add if more adherends and adhesive layers are needed
t = [t1; t2; t3];
e = [e1; e2; e3; e4];

e_ME = zeros(P*n_ME,1);  % vector of the adherend thickness(mm)
for i=1:P
    for j=1:n_ME
        e_ME(i+(j-1)*P,1) = e(i);
    end
end 


t_ME = zeros((P-1)*n_ME,1);  % vector of the adhesive thickness (mm)
for i=1:P-1
    for j=1:n_ME
        t_ME(i+(j-1)*(P-1),1)=t(i);
    end
end  

Ltot = L;      % overlap length      (mm)

L_ME = zeros(n_ME,1);
for i = 1:n_ME
    L_ME(i,1) = Ltot/n_ME;    %ME length (regular mesh)
end
     
abscissa = zeros(n_ME+1,1);
for i=2:n_ME+1
    abscissa(i,1)=abscissa(i-1,1)+L_ME(i-1,1);
end

% =========================================================================
% A3 - MATERIAL
% =========================================================================
% Material Properties
% E Young modulus of adherend (MPa)
% G Shear modulus of adhesive (MPa)
% =========================================================================
% A3.1 - ADHERENDS
% =========================================================================
% Add if more adherends and adhesive layers are needed
E1 = 70000;       %MPa
E2 = 70000;       %MPa
E3 = 70000;       %MPa
E4 = 70000;       %MPa
% E5 = 70000;       %MPa
% E6 = 70000;       %MPa
% E7 = 70000;       %MPa
% E8 = 70000;       %MPa
% E9 = 70000;       %MPa
% E10 = 70000;      %MPa

% Add if more adherends and adhesive layers are needed
E = [E1; E2; E3; E4];

E_ME = zeros(P*n_ME,1);  % vector of the Young's modulus (MPa)

for i=1:P
    for j=1:n_ME
        E_ME(i+(j-1)*P,1) = E(i);
    end
end 

% =========================================================================
% A3.2 - ADHESIVES
% =========================================================================
% Add if more adherends and adhesive layers are needed
G1 = 100;         %MPa
G2 = 100;         %MPa
G3 = 100;         %MPa
% G4 = 100;         %MPa
% G5 = 100;         %MPa
% G6 = 100;         %MPa
% G7 = 100;         %MPa
% G8 = 100;         %MPa
% G9 = 100;         %MPa
% G10 = 100;        %MPa

% Add if more adherends and adhesive layers are needed
G = [G1; G2; G3];

G_ME = zeros((P-1)*n_ME,1);  % vector of the adhesive shear modulus (MPa)
for i=1:P-1
    for j=1:n_ME
        G_ME(i+(j-1)*(P-1),1) = G(i);
    end
end 

% =========================================================================
% A4 - BC & LOADING
% =========================================================================
% =========================================================================
% A4.1 - LOADING
% =========================================================================

F_load = 100.;                             % applied force in N
% =========================================================================
% A - END OF PROBLEM DEFINITION
% =========================================================================
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% ========================================================================
% B - INITIAL STIFFNESS MATRIX ASSEMBLY
% ========================================================================
% =========================================================================
% B1 - ELEMENTARY STIFFNESS MATRICES
% =========================================================================
% =========================================================================
% B1.1 - MACRO-ELEMENT
% =========================================================================
% K_Bar = zeros(2*P,2*P,n_ME);
% M_Bar = zeros(2*P,2*P,n_ME);
% N_Bar = zeros(2*P,2*P,n_ME);
[K_Bar, M_Bar, N_Bar] = RESOLME_Bar_expm(P, n_ME, L_ME, G_ME, e_ME, b, t_ME, E_ME);
% =========================================================================
% B2 - STIFFNESS MATRIX
% =========================================================================
% =========================================================================
% B2.1 ASSEMBLY OF THE STIFFNESS MATRIX OF THE OVERLAP
% =========================================================================
% KK = zeros(2*P,2*P);
% Assembly of the Elementary Stiff. Matrices (K_Bar) of Bonded Bar Elements
% =========================================================================
% B2.2 ASSEMBLY OF THE STIFFNESS MATRIX OF THE SPECIMEN
% =========================================================================
% Ktot =K
Ktot = zeros(n_DOF,n_DOF);

for k=1:n_ME
    for i=1:P    
        for j=1:P
            Ktot((k-1)*P+i,(k-1)*P+j) = K_Bar(2*i-1,2*j-1,k)+Ktot((k-2)*P+P+i,(k-2)*P+P+j);
            Ktot((k-1)*P+i,(k-1)*P+P+j) = K_Bar(2*i-1,2*j,k);
            Ktot((k-1)*P+P+i,(k-1)*P+j) = K_Bar(2*i,2*j-1,k);
            Ktot((k-1)*P+P+i,(k-1)*P+P+j) = K_Bar(2*i,2*j,k);
        end 
    end
end 


% =========================================================================
% B2 - AUGMENTED STIFFNESS MATRIX
% =========================================================================           
 max_AUG = n_DOF+P;
 CLaug = zeros(P,n_DOF);  
for i = 1:P                  %u1=u2=..uP=0, clamped
    CLaug(i,i) = 1;
end  
    
Kaug = zeros(n_DOF+P,n_DOF+P);
Kaug(1:n_DOF,1:n_DOF) = Ktot;
Kaug(n_DOF+1:n_DOF+P,1:n_DOF) = CLaug;
Kaug(1:n_DOF,n_DOF+1:n_DOF+P) = CLaug';
    
qL = zeros(P,1);
FL = zeros(n_DOF+P,1);
FL(n_DOF,1) = FL(n_DOF,1) + F_load;
%     The force loading can be applied to different adherend like given
%     example line for 3rd adherend layer
%     ==> FL(n_DOF-1,1) = FL(n_DOF-1,1) + F_load; 
FL(n_DOF+1:n_DOF+P,1) = qL;


% =========================================================================
% B2 - BOUNDARY CONDITIONS & LOADING
% =========================================================================
% Boundary conditions:
% Multilayered Bar Element is clamped at x=0.
% Therefore, u(1:P)=0.
% =========================================================================
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% ========================================================================
% C - RESOLUTION & POSTPROCESSING
% ========================================================================
% =========================================================================
% C1 - SOLVER & POSTPROCESSING
% =========================================================================
% C1.1 - RESULTS FOR THE ADHERENDS ALONG THE OVERLAP
% ========================================================================= 
% First P elements of the vector are too small, nearly equal to zero 
% since all the layers are clamped at x=0.
% Result of XL:
% First n_DOF elements of the vector correspond to the displacements of each DOFs
% Last P elements (n_DOF+1:n_DOF+P) of the vector correspond to the
% reaction forces on the clamped ends due to the applied force to the structure.
% Since XL=displacements+reactions, sum(reactions of XL)=0.

XL = Kaug\FL;

% =========================================================================
% C1.2 - RESULTS FOR THE ADHERENDS ALONG THE OVERLAP
% =========================================================================
% Shear modulus and adhesive thickness for ME %

% It is assumed that each adhesive layer (from 1 to n_ME)
% has same material properties (G) and thickness along the one abscissa.
% G(1) = G(1+1*n_ME) = G(1+2*n_ME) = .... = G(1+(P-2)*n_ME) 
% G(2) = G(2+1*n_ME) = G(2+2*n_ME) = .... = G(2+(P-2)*n_ME)
% ........
% G(p-1) = G((p-1)+1*n_ME) = G((p-1)+2*n_ME) = .... = G((p-1)+(P-2)*n_ME)
% Same algorithm is valid for the adhesive thickness t(i).

U = XL(1:n_DOF,1);

G_MME = zeros((P-1)*n_ME,1);
t_MME = zeros((P-1)*n_ME,1);

for i=1:P-1
    for j=1:n_ME
        G_MME(i+(P-1)*(j-1),1) = G(i,1);
        t_MME(i+(P-1)*(j-1),1) = t(i,1);
    end
end 

% Shear Stress %%%%

T = zeros((P-1)*n_ME*2,1);

for i  = 1:n_ME
    for j = 1:(P-1)
        T(2*j-1+(P-1)*2*(i-1),1) = (G_MME((j+(i-1)*(P-1)),1)/t_MME((j+(i-1)*(P-1)),1))*(U((i-1)*P+j+1,1)-U((i-1)*P+j,1));
        T(2*j+(P-1)*2*(i-1),1) = (G_MME((j+(i-1)*(P-1)),1)/t_MME((j+(i-1)*(P-1)),1))*(U(((i-1)*P+j+1+P),1)-U(((i-1)*P+j+P),1));
    end 
end 


% Reduced Shear Stress Vector %%%%%

% The matrix is needed to be reduced since the same value for G and t is used
% along the abscissa of each adhesive layer. (linear adhesive properties)
% It means that for example, such bonded joint, having
% P=4 adherend layers, P-1=3 adhesive, nME=3 macro-element,
% for adhesive in first macro-element ;
% (which has two nodes: T1(0) and T1(L/ME)) 
% and adhesive in (1+P-1) 4th macro-element 
% (which has two nodes: T4(0)=T4(L/ME))
% T1(L/ME) = T4(0)
% because 
% adhesive in 1st macro-element and adhesive in (1+P-1) 4th macro element
% are along the same abscissa according to the way of mesh numbering.
% After reducing the matrix, there are (nME+1) T values along the
% abscissa.

Tred = zeros((P-1),n_ME+1);

%Non-reduced part (first and last (n_ME+1) nodes of each adhesive layer)
for i = 1:P-1
    Tred(i,1) = T(2*i-1,1);
    Tred(i,n_ME+1) = T(2*(n_ME-1)*(P-1)+2*i,1);
end 

% Reduced part      (coincident nodes)
for i = 1:n_ME-1
    for j = 1:(P-1)
        Tred(j,i+1) = T(2*j+(P-1)*2*(i-1),1);
    end 
end

% For the analysis:
Tshear = transpose(Tred);

%==========================================================================
% CURVES
%==========================================================================
% Adhesive Shear Stress
figure
plot(abscissa,transpose(Tred(1,1:n_ME+1)),'DisplayName','1st Adhesive Layer')
hold on 
plot(abscissa,transpose(Tred(2,1:n_ME+1)),'DisplayName','2nd Adhesive Layer')
hold on 
plot(abscissa,transpose(Tred(P-1,1:n_ME+1)),'DisplayName','3rd Adhesive Layer')
axis([0 L 0 6])
grid on
box on
hold off
title('Adhesive shear stress distributions along the overlap')
xlabel('Abscissa along the overlap in mm')
ylabel('Adhesive Shear Stress in MPa')

legend