% Author: Kubra SEKMEN* & Eric PAROISSIEN & Frederic LACHAUD
% ISAE - SUPAERO Institut Suprieur de l'Aronautique et de l'Espace
% Date: 10.07.2019
% Paper Title: Simplified Stress Analysis of Multilayered Adhesively
%               Bonded Structures
% Code attached to the related paper submitted to AB2019
% ==> 1D Beam Model - Goland & Reissner Simplified Hypotheses
% ==> in-plane tensile loading at the tip (x=L) of the last adherend layer
% LICENSE: CC BY NC 3.0 [https://creativecommons.org/licenses/by-nc/3.0/]
%==========================================================================
warning ('off');
clc;
clear all;
close all;
tic
% ========================================================================
% A - PROBLEM DEFINITION
% ========================================================================
% Model of beam for layers: Euler-Bernoulli beam model

% DEFINITION OF THE GEOMETRICAL TYPE
% multilayered specimen
% clamped at x=0 & free end at x=L
% =========================================================================

% =========================================================================
% A1 - SIZE OF THE PROBLEM
% =========================================================================
n_P_ME = 4;     % number of layers
n_ME = 100;     % number of macro-elemenT

% DEDUCED PARAMETERS
[n_nodes_ME, n_DOF_ME] = SizeModel(n_P_ME, n_ME);
% =========================================================================

% =========================================================================
% A2 - GEOMETRY
% =========================================================================
Ltot = 30;   % overlap length in mm
btot = 1;    % width of ME in mm

e_ME = zeros(n_P_ME,n_ME);                     % thickness of ME-layers in mm
ea_ME = zeros(n_P_ME,n_ME);                    % adhesive thickness of ME-layers in mm
for i=1:n_P_ME                                 % Adherend thickness in mm
    for j=1:n_ME
        e_ME(i,j) = 2.5;
    end
end 

for i=1:n_P_ME                                 % Adhesive thickness in mm
    for j=1:n_ME
        ea_ME(i,j) = 0.11;
    end
end

% DEDUCED PARAMTERS
[L_ME,b_ME,h_ME,S_ME,I_ME] = Geometry(Ltot,btot,e_ME,n_ME,n_P_ME);
for j=1:n_ME
            L_ME(1,j) = Ltot/n_ME;
end

[abscissa_ME,abscissaCentro_ME] = Abscissa(L_ME,n_ME);
% =========================================================================

% =========================================================================
% A3 - MATERIAL
% =========================================================================
% =========================================================================
% A3.1 - LAYERS
% =========================================================================
Er_ME = zeros(n_P_ME,n_ME);              % Young's modulus in MPa of ME-layers 

for i=1:n_P_ME
    for j=1:n_ME
        Er_ME(i,j) = 70000;
    end
end

% DEDUCED PARAMETERS ------------------------------------------------------ 
[Ar_ME,Dr_ME]=MaterialStiffness(Er_ME,S_ME,I_ME,n_ME,n_P_ME);

% =========================================================================
% A3.2 - INTERFACES
% =========================================================================
t_ae = 0.11;                          % adhesive thickness "ea" (mm)
G_ae = 100;                           % adhesive shear modulus (MPa)
nu_ae = 0.33;                         % adhesive Poisson's ratio
E_ae = 2*G_ae*(1+nu_ae);              % adhesive Young's modulus (MPa)
kn = E_ae/t_ae;                       % adhesive elastic stiffness in peel (Eadhesive/t) in MPA/mm
ks = G_ae/t_ae;                       % adhesive elastic stiffness in shear (Gadhesive/t) in MPA/mm

kIa_ME = zeros(n_P_ME-1,n_ME);            % interface peel modulus in MPa.mm = N/mm   
kIIa_ME = zeros(n_P_ME-1,n_ME);           % interface shear modulus in MPa.mm = N/mm
kI_ME = zeros(n_P_ME-1,n_ME);             % interface peel modulus in MPa.mm = N/mm 
kII_ME = zeros(n_P_ME-1,n_ME);            % interface peel modulus in MPa.mm = N/mm 
for i=1:n_P_ME-1
    for j=1:n_ME
        kIa_ME(i,j) = kn;
        kIIa_ME(i,j) = ks;
    end
end
for i=1:n_P_ME-1
    for j=1:n_ME
        kI_ME(i,j) = kIa_ME(i,j);
        kII_ME(i,j) = kIIa_ME(i,j);
    end
end

% =========================================================================

% =========================================================================
% A4 - FORMULATION HYPOTHESES
% =========================================================================
% SYM: symmetrical part of K_elem
% -------- SYM=0: non activation of symmetrical part of K 
% -------- SYM=1: activation of symmetrical part of K 
SYM=1;
% =========================================================================

% =========================================================================
% A5 - LOADING
% =========================================================================   
Fload=100.;                             % applied force in N
% =========================================================================


%==========================================================================
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% =========================================================================
% B - INITIAL STIFFNESS MATRIX
% =========================================================================

% =========================================================================
% B1 - ELEMNTARY STIFFNESS MATRICES
% =========================================================================
[K_ME,M_ME,N_ME] = MEXGR(n_ME,n_P_ME,b_ME,h_ME,kI_ME,kII_ME,Ar_ME,Dr_ME,L_ME);

if SYM==1
    for i=1:n_ME
        K_ME(:,:,i)=0.5*(K_ME(:,:,i)+K_ME(:,:,i).');
    end
end

% =========================================================================

% =========================================================================
% B2 - STRUCTURE STIFFNESS MATRICES
% =========================================================================
% Siffness matrix assembly for each lay-up from elementary stiffness matrices
[KK_ME] = AssemblageLap(K_ME,n_ME,n_P_ME);


% Siffness matrix assembly for structure from each lay-up
[Ktot] = AssemblageStruct(KK_ME);

%==========================================================================
% B3 - STRUCTURE AUGMENTED STIFFNESS MATRICES WITH PRESCRIBED AND FIXED DOF
%==========================================================================
[maxAUG] = maxAUGMENTE(n_P_ME,n_DOF_ME);
[Kaug] = AssemblageStructAUG(Ktot,n_ME,n_P_ME,n_DOF_ME,n_nodes_ME);

% =========================================================================
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% =========================================================================
% C - SOLVER
% =========================================================================
% Loading (augmented force vector) 
f = Fload;
[FL] = LoadAugFor(f,n_ME,n_P_ME,n_DOF_ME);
Xcumul = zeros(maxAUG,1);  

Xcumul = zeros(maxAUG,1);
Xcumul=Kaug\FL;
        
% Elastic prediction               
Fext=Kaug*Xcumul; 
    
% Post processing     
XXpost(1:n_DOF_ME,1)=Xcumul(1:n_DOF_ME,1);
[Xtot,resLAYER_ME,resINTERFACE_ME,resSPECIMEN,w_ME,wg_ME,wd_ME,eps_ME,U_ME] = PostProcess(XXpost,n_ME,n_P_ME,kI_ME,kII_ME,h_ME,K_ME,M_ME);

% Peel stress results
S_i = zeros(n_ME+1, n_P_ME-1);
S_i(1,1:n_P_ME-1)=0;
S_i(2:n_ME+1,1:n_P_ME-1)= resINTERFACE_ME(1:n_ME,n_P_ME+1:2*n_P_ME-1);
% Shear stress results
T_i = zeros(n_ME+1, n_P_ME-1);
T_i(1,1:n_P_ME-1)=0;
T_i(2:n_ME+1,1:n_P_ME-1)= resINTERFACE_ME(1:n_ME,2*n_P_ME+1:3*n_P_ME-1);

% Adhesive Shear Stress
figure
plot(abscissa_ME,transpose(T_i(1:n_ME+1,1)),'DisplayName','1st Adhesive Layer')
hold on 
plot(abscissa_ME,transpose(T_i(1:n_ME+1,2)),'DisplayName','2nd Adhesive Layer')
hold on 
plot(abscissa_ME,transpose(T_i(1:n_ME+1,3)),'DisplayName','3rd Adhesive Layer')
axis([0 Ltot -1 4])
grid on
box on
hold off
title('Adhesive shear stress distributions along the overlap')
xlabel('Abscissa along the overlap in mm')
ylabel('Adhesive Shear Stress in MPa')

legend


% Adhesive Peel Stress
figure
plot(abscissa_ME,transpose(S_i(1:n_ME+1,1)),'DisplayName','1st Adhesive Layer')
hold on 
plot(abscissa_ME,transpose(S_i(1:n_ME+1,2)),'DisplayName','2nd Adhesive Layer')
hold on 
plot(abscissa_ME,transpose(S_i(1:n_ME+1,3)),'DisplayName','3rd Adhesive Layer')
axis([0 Ltot -2.5 0.5])
grid on
box on
hold off
title('Adhesive peel stress distributions along the overlap')
xlabel('Abscissa along the overlap in mm')
ylabel('Adhesive Peel Stress in MPa')

legend
